package org.one.stone.soup.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.stream.ImageInputStream;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileFilter;

import org.one.stone.soup.grfx.ImageFactory;
import org.one.stone.soup.stringhelper.StringGenerator;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ImagePreviewAccessory extends JPanel implements ActionListener,PropertyChangeListener
{
	private JFileChooser chooser;

	private ImageIcon icon;
	private JLabel label;

	private JLabel size;
	private JLabel name;


	public ImagePreviewAccessory(JFileChooser chooser)
	{
		this.chooser = chooser;

		this.setLayout(new BorderLayout(5,5));
		this.setBackground(Color.darkGray);
		this.setForeground(Color.white);

		name = new JLabel("",SwingConstants.CENTER);
		name.setForeground(Color.white);
		add(name,BorderLayout.NORTH);

		icon = new ImageIcon( ImageFactory.createVGradImage(Color.black,Color.darkGray,100,100) );
		label = new JLabel(icon);
		label.setBorder( BorderFactory.createLineBorder(Color.white,1) );
		add(label,BorderLayout.CENTER);

		size = new JLabel("w: h:",SwingConstants.CENTER);
		size.setForeground(Color.white);
		add(size,BorderLayout.SOUTH);
	}

	public void propertyChange(PropertyChangeEvent p)
	{
		File file = chooser.getSelectedFile();

		if(file!=null)
		{
			if(file.isDirectory())// || filter.accept(file)==false)
			{
				return;
			}

			FileFilter filter = chooser.getFileFilter();
			if(filter instanceof SimpleFileFilter)
			{
				if(((SimpleFileFilter)filter).accept(file)==false)
				{
					return;
				}
			}
			else
			{
				return;
			}

			String fileName = file.getAbsolutePath();
			System.out.println("loading");
			Image image = ImageFactory.loadImage(fileName);
			System.out.println("loaded");

			int w = image.getWidth(this);
			int h = image.getHeight(this);
			double f = 1;

			size.setText("w:"+image.getWidth(this)+" h:"+image.getHeight(this));

			name.setText("  "+file.getName()+"  ");

			if(w>100)
			{
				f = 100.0/w;
			}

			if(h>100 && w<h)
			{
				f = 100.0/h;
			}

			w = (int)(w*f);
			h = (int)(h*f);

			int offX = (100-w)/2;
			int offY = (100-h)/2;

			if(f<1)
			{
				System.out.println("scaleing");				
				image = image.getScaledInstance(w,h,Image.SCALE_SMOOTH);
				System.out.println("scaled");				
				icon.setImage(image);
			}
			else
			{
				Image showImage = createImage(100,100);
				Image imageBacking = ImageFactory.createVGradImage(Color.black,Color.darkGray,100,100);

				showImage.getGraphics().drawImage(imageBacking,0,0,this);
				showImage.getGraphics().drawImage(image,offX,offY,w,h,this);

				icon.setImage(showImage);
			}

			try{
				ImageInputStream stream = ImageIO.createImageInputStream( new File(fileName) );
				ImageReader reader = (ImageReader)ImageIO.getImageReaders(stream).next();//ByFormatName( "png" ).next();
				reader.setInput(stream);
				IIOMetadata metadata=reader.getImageMetadata(0);
				String name = metadata.getNativeMetadataFormatName();
				Node node = reader.getImageMetadata(0).getAsTree(name);

				String data = getMetaData( node );
				System.out.println("MetaData:\n"+data);

				label.setToolTipText( data );
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			chooser.repaint();
		}
	}

	public String getMetaData( Node node )
	{
		StringBuffer buffer = new StringBuffer();

		createMetaData(node,buffer,0,": ");

		return buffer.toString();
	}

	public void createMetaData(Node node,StringBuffer buffer,int level,String spacer)
	{
		buffer.append( StringGenerator.generatePad(level,'\t') );
		buffer.append(node.getNodeName());
		buffer.append( spacer );
		buffer.append(node.getNodeValue());
		buffer.append("\n");

		NamedNodeMap map = node.getAttributes();
		for(int loop=0;loop<map.getLength();loop++)
		{
			createMetaData( map.item(loop),buffer,level+1," = " );
		}

		NodeList list = node.getChildNodes();
		for(int loop=0;loop<list.getLength();loop++)
		{
			createMetaData( list.item(loop),buffer,level+1,": " );
		}
	}

	public void actionPerformed(ActionEvent a)
	{
	}
/*
	public Dimension getSize()
	{
		return new Dimension(150,150);
	}

	public Dimension getMinimumSize()
	{
		return getSize();
	}
	public Dimension getMaximumSize()
	{
		return getSize();
	}
	public Dimension getPreferedSize()
	{
		return getSize();
	}

	public void setBounds(int x,int y,int width,int height)
	{
		super.setBounds(x,y,150,150);
	}*/
}
