package org.one.stone.soup.swing;

import java.awt.*;
import javax.swing.*;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;

import org.one.stone.soup.stringhelper.StringArrayHelper;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */
public class JTextConsole extends JScrollPane {

	private Color[] highlightColor = new Color[]{
											new Color(0,0,150),
											new Color(0,150,0),
											new Color(150,0,0),
											new Color(120,120,0),
											new Color(120,0,120),
											new Color(0,120,120),
											new Color(120,0,120)
										};

	private JTextArea textArea;

public JTextConsole()
{
	this(20,80);
}
public JTextConsole(int rows, int columns) {
	textArea = new JTextArea(rows, columns);
	textArea.setEditable(true);

	setViewportView( textArea );
}
/**
 *
 */
public void clear()
{
	textArea.setText("");
}

public String getSelected()
{
	return textArea.getSelectedText();
}

public void find(String find)
{
	String[] phrases = StringArrayHelper.parseSpacedFields( find );

	Highlighter highlighter = textArea.getHighlighter();
	highlighter.removeAllHighlights();

	for(int loop=0;loop<phrases.length;loop++)
	{
		find = phrases[loop];
		Color color = highlightColor[loop%highlightColor.length];

		String text = textArea.getText();

		int index = text.indexOf(find);
		int pos = 0;

		while(index!=-1)
		{
			try{
				highlighter.addHighlight(index,index+find.length(),new DefaultHighlighter.DefaultHighlightPainter(color));
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			pos=index+find.length();
			index = text.indexOf(find,pos);
			//CodeHighlight highlight = new CodeHighlight(textArea,index,index+find.length(),Color.green);
		}
	}
}

public void replace(String find,String replace)
{
	String text = textArea.getText();

	text = text.replaceAll( find,replace );

	textArea.setText(text);
}

public void setBackground(Color c)
{
	super.setBackground(c);

	if(textArea!=null)
	{
		textArea.setBackground(c);
	}
}
public void setForeground(Color c)
{
	super.setForeground(c);

	if(textArea!=null)
	{
		textArea.setForeground(c);
		textArea.setCaretColor(c);
	}
}

public void setEnabled(boolean editable)
{
	textArea.setEditable(editable);
}

public String getText()
{
	return textArea.getText();
}

public void setText(String text)
{
	textArea.setText(text);
}

public void appendText(String newText)
{
	textArea.setText( textArea.getText()+newText );
}

public Component getComponent()
{
	return textArea;
}

}

