package org.one.stone.soup.xapp;

import java.awt.Dimension;
import java.io.File;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Vector;

import org.one.stone.soup.browser.Browser;
import org.one.stone.soup.browser.cache.SimpleResourceStore;
import org.one.stone.soup.exception.ExceptionHelper;
import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.grfx.ScreenCapture;
import org.one.stone.soup.reloader.ReloadListener;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.util.Queue;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.compound.XappSplashPage;
import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.containers.XappContainer;
import org.one.stone.soup.xapp.containers.XappMenuBar;
import org.one.stone.soup.xapp.dnd.XappDragSource;
import org.one.stone.soup.xapp.dnd.XappDropReceiver;
import org.one.stone.soup.xapp.factories.XappComponentFactory;
import org.one.stone.soup.xapp.factories.XappPluginFactory;
import org.one.stone.soup.xapp.javascript.JavascriptEngine;
import org.one.stone.soup.xapp.templates.XappSchemeToXappThemeConverter;
import org.one.stone.soup.xapp.xpath.XappXPathDB;
import org.one.stone.soup.xapp.xpath.XappXPathDBChangeListener;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.XmlParser;
import org.one.stone.soup.xml.stream.XmlLoader;

public abstract class XApplication implements XappSender,XappReceiver,XappXPathDBChangeListener,ReloadListener{

	private boolean rootApplication = true;
	private boolean autoSave = true;

	private XappComponentFactory componentFactory;
	private XappApplicationFrame frame;
	private XappSplashPage splashPage;	
	
	private JavascriptEngine engine;
	private XappPluginStore pluginStore = new XappPluginStore();
	private Vector persist;
	private String title;
	private String definitionFileName;
	private Dimension preferredSize = new Dimension(400,400);
	
	private XappComponentStore componentStore = new XappComponentStore();
	private XappXPathDB xAppConfig;

	private Queue transactionQueue = new Queue();
	
	private class QueueMonitor implements Runnable
	{	
		public boolean running = false;
		public QueueMonitor(String alias)
		{
			new Thread(this,alias).start();
		}
		
		public void run()
		{
			running = true;
			while(running)
			{
				try{
					Thread.sleep(10);
					
					while( !transactionQueue.isEmpty())
					{
						XmlElement transaction = (XmlElement)transactionQueue.get();
						
						processTransaction(transaction);
						queuing();
					}
				}
				catch(Exception e)
				{
					//displayException(e);
					e.printStackTrace();
				}
			}
			running = false;
		}
	}
	private QueueMonitor queueMonitor;	
	
	
	public XApplication(String xappDefinitionFile) throws Exception 
	{
		
		XappRootApplication.setRoot(this);

		try{
			if( xappDefinitionFile==null )//|| !new File(xappDefinitionFile).getAbsoluteFile().exists() )
			{
				throw new Exception("Definition file is null");
			}
			if(xappDefinitionFile.indexOf(':')<2)
			{
				definitionFileName = new File(xappDefinitionFile).getAbsolutePath();
			}
			else
			{
				definitionFileName = xappDefinitionFile;
			}
			String key = Browser.initialize(false,"Xapp");
			Browser.releaseKey(key);
			Browser.setResourceStore( new SimpleResourceStore() );
			System.setProperty("user.dir","");
			Browser.setBaseURL(getApplicationFileRoot(),null);
			System.setProperty("user.dir",getApplicationFileRoot());
			xAppConfig = new XappXPathDB( definitionFileName,this);

			String renderer = xAppConfig.findElement("xapp/view").getAttributeValueByName("renderer");
			if(renderer==null)
			{
				renderer = "org.one.stone.soup.xapp.swing.factories.XappSwingComponentFactory";
			}
			
			componentFactory = (XappComponentFactory)Class.forName(renderer).newInstance();
			frame = componentFactory.buildApplicationFrame();
			componentStore = new XappComponentStore(frame);
			
		}
		catch(Exception e)
		{			
			if(isRootApplication()==true)
			{
				if(componentFactory!=null)
				{
					componentFactory.closeApplication(this);
				}
			}
			else
			{
				queueMonitor.running=false;
				frame.dispose();
			}
			
			throw e;
		}
		
		prepare();
	}
	
	public String getApplicationFileRoot()
	{
		if(definitionFileName.indexOf(':')>=2)
		{
			String root = definitionFileName;
			if(root.lastIndexOf('/')>-1)
			{
				root = root.substring(0,root.lastIndexOf('/')+1);
			}
			return root;
		}
		else
		{
			return new File(definitionFileName).getParent()+"/";
		}
	}
	
	public String convertFileNameToRelative(String fileName)
	{
		if( fileName.indexOf(getApplicationFileRoot())==0 )
		{
			return fileName.substring( getApplicationFileRoot().length() );
		}
		else
		{
			return fileName;
		}
	}
	
	public void initialise()
	{
		//prepare();
		
		if(splashPage!=null)
		{
			splashPage.dispose();
			splashPage=null;
		}
		
		XmlElement controller = xAppConfig.getRoot().getElementByName("controller");

		persist = controller.getElementsByName("persistXappData");
		if(persist.size()!=0)
		{
			retrievePersistedData();
		}
		
		pluginStore = new XappPluginStore();
		Vector pluginFolders = controller.getElementsByName("pluginFolder");
		for(int loop=0;loop<pluginFolders.size();loop++)
		{
			XmlElement xPluginFolder = (XmlElement)pluginFolders.elementAt(loop);
			loadPlugins( xPluginFolder.getValue(),controller );
		}
		
		initialisePlugins();
		
		frame.setSize( preferredSize );
		frame.show();
	}
	
	public void reinitialise()
	{
		try{
			prepare();
		}
		catch(Exception e)
		{
			XappRootApplication.displayException(e);
			if( destroy()==true )
			{
				componentFactory.closeApplication(this);
			}
		}
		
//		pack();
		frame.setSize(new Dimension(preferredSize));
		frame.show();
	}
	
	private void prepare()
	{	
		buildApplication();

		XmlElement xController = xAppConfig.getRoot().getElementByName("controller");
		buildController(xController);
		
		if(queueMonitor==null)
		{
			queueMonitor = new QueueMonitor("Xapp Queue monitor");
		}
		
		engine = new JavascriptEngine();		
	}

	public String getTitle()
	{
		return title;
	}
	
	private void buildApplication()
	{
		title = xAppConfig.getRoot().getAttributeValueByName("title");
		
		if(title==null)
		{
			try{
				Method method = this.getClass().getMethod("setUndecorated",new Class[]{boolean.class});
				method.invoke( this,new Object[]{new Boolean(true)} );
			}
			catch(Throwable e)
			{
				e.printStackTrace();
				System.out.println("XApplication.setUndecorated not available in JVM");
			}
		}
		else
		{
			frame.setTitle(title);
		}
		
		XmlElement xTheme = xAppConfig.getRoot().getElementByName("xTheme");
		if(xTheme!=null)
		{
			frame.setTheme(xTheme);
		}
		else
		{
			xTheme = xAppConfig.getRoot().getElementByName("xScheme");	
			if(xTheme!=null)
			{
				frame.setTheme(xTheme);
			}
		}		
		
		XmlElement splash = xAppConfig.getRoot().getElementByName("splash");
		if(splash!=null)
		{
			splashPage = XappRootApplication.getComponentFactory().buildSplashPage(splash);
		}
		
		XmlElement xMenuBar = xAppConfig.getRoot().getElementByName("menuBar");
		if(xMenuBar!=null)
		{
			XappMenuBar menuBar = XappRootApplication.getComponentFactory().buildXappMenuBar(xMenuBar,componentStore);
			menuBar.addReceiver(this);
			frame.setXappMenuBar(menuBar);
		}
		
		XmlElement xView = xAppConfig.getRoot().getElementByName("view");
		buildView( xView );

	}	
	
/*	public static XApplication getMainApplication()
	{
		return main;
	}*/
	
	public final void exit()
	{
		if(destroy()==true)
		{
			if(isRootApplication())
			{
				componentFactory.closeApplication(this);
			}
			else
			{
				frame.dispose();
			}
		}
	}
	
	public boolean codeHasChangedReload()
	{
		if( frame.askForConfirmation("The application code has changed. Reload ?") == true)
		{
			savePersistedData();
			frame.dispose();
			return true;
		}
		else
		{
			return false;
		}
	}
	
public boolean destroy()
{
	if(persist!=null && persist.size()!=0 && frame.askForConfirmation("Save Application Data")==true )
	{	
		savePersistedData();
	}
	
	if( frame.askForConfirmation( "Exit Application")==true )
	{	
		if(isRootApplication())
		{
			return true;
		}
		else
		{
			frame.dispose();
			return false;
		}
	}
	else
	{
		return false;
	}
}

public void receiveXappData(XmlElement data) {
	transactionQueue.post(data);
	queuing();
}
public void processTransaction(XmlElement data)
{
	if(data.getName().equals("action"))
	{
		String name = data.getElementByIndex(0).getName();
		String command = data.getElementByIndex(0).getValue();
		
		processAction( name,command );
	}
	else if(data.getName().equals("treeAction"))
	{
		boolean rightButton = false;
		if(data.getAttributeValueByName("button").equals("right"))
		{
			rightButton = true;
		}
		int x = Integer.parseInt( data.getAttributeValueByName("x") );
		int y = Integer.parseInt( data.getAttributeValueByName("y") );
		
		processTreeAction( data.getElementByIndex(0),data.getElementByIndex(1),rightButton,x,y );
	}
	else if(data.getName().equals("drop"))
	{
		String file = data.getElementByName("file").getValue();
		
		int x = Integer.parseInt( data.getAttributeValueByName("x") );
		int y = Integer.parseInt( data.getAttributeValueByName("y") );

		processDropAction( file,x,y );
	}
	else if(data.getName().equals("popup"))
	{
		processPopupAction( data );
	}
	else
	{
		processPost( data );
	}
	
}

public abstract void processDropAction( String file,int posX,int posY );
public abstract void processTreeAction( XmlElement dataRoot,XmlElement dataSource,boolean rightButton,int posX,int posY );
public abstract void processPopupAction( XmlElement data );
public abstract void processPost(XmlElement data);
public boolean processAction(String sourceName,String actionName)
{
	String[] args = StringArrayHelper.parseFields( actionName,' ','\"','\"' );
	actionName = args[0];
	
	if(actionName.equals("javascript:"))
	{
		String script = StringArrayHelper.arrayToString(args," ",1,args.length);
		engine.runJavascript(script);
		
		return true;
	}
	
	XmlElement action = xAppConfig.findElement("xapp/controller/action[@name='"+actionName+"']");
	
	if(action!=null)
	{
		String method = action.getAttributeValueByName("method");
		if(method==null)
		{
			method = actionName;
		}
		String pluginAlias = action.getAttributeValueByName("plugin");		
		String adapter = action.getAttributeValueByName("adapter");
		if(pluginAlias!=null)
		{
			XappPlugin plugin = pluginStore.getPlugin( pluginAlias );
			
			try{
				Object result = null;
				if(args.length>1)
				{
					result = plugin.getClass().getMethod(method,new Class[]{String[].class}).invoke(plugin,new Object[]{args});					
				}
				else
				{
					result = plugin.getClass().getMethod(method,new Class[]{}).invoke(plugin,new Object[]{});
				}
				
				if(result instanceof Boolean)
				{
					return ((Boolean)result).booleanValue();
				}
				else
				{
					return true;
				}
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
				return false;
			}			
		}
		else if(adapter!=null)
		{
			XmlElement xActionAdapter = xAppConfig.findElement("xapp/controller/actionAdapter[@name='"+adapter+"']");
			String className  = xActionAdapter.getAttributeValueByName("class");
			
			try{
				Object actionAdapter = Class.forName(className).getConstructor(new Class[]{XmlElement.class}).newInstance(new Object[]{xActionAdapter});
				Object result = null;
				if(args.length>1)
				{
					Class.forName(className).getMethod(method,new Class[]{String[].class}).invoke(actionAdapter,new Object[]{args});					
				}
				else
				{
					Class.forName(className).getMethod(method,new Class[]{}).invoke(actionAdapter,new Object[]{});
				}
				
				if(result instanceof Boolean)
				{
					return ((Boolean)result).booleanValue();
				}
				else
				{
					return true;
				}
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
				return false;
			}			
		}
		else
		{
			try{
				Object result = null;
				if(args.length>1)
				{
					result = this.getClass().getMethod(method,new Class[]{String[].class}).invoke(this,new Object[]{args});					
				}
				else
				{
					result = this.getClass().getMethod(method,new Class[]{}).invoke(this,new Object[]{});
				}
				
				if(result instanceof Boolean)
				{
					return ((Boolean)result).booleanValue();
				}
				else
				{
					return true;
				}
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
				return false;
			}
		}
	}
	frame.showMessage("Action not found "+actionName);
	
	return false;
}

private void retrievePersistedData()
{
	for(int loop=0;loop<persist.size();loop++)
	{
		XmlElement entry = (XmlElement)persist.elementAt(loop);
		
		String inFile = getApplicationFileRoot()+entry.getAttributeValueByName("in");
		if(inFile.indexOf(':')<2 && !new File( inFile ).exists())
		{
			continue;
		}
		
		XmlParser parser = new XmlParser();
		parser.removeImportParser();
		XmlElement data = null;
		try{
			if(inFile.indexOf(':')>=2)
			{
				InputStream iStream = Browser.getInputStream(inFile);
				if(iStream==null)
				{
					continue;
				}
				data = parser.parseElement( iStream );
			}
			else
			{
				data = parser.parseElement( new File(inFile).getAbsoluteFile() );
			}
		}
		catch(Exception e)
		{
			_displayException(e);
		}
		if(data==null)
		{
			return;
		}
		
		setData(entry.getAttributeValueByName("from"),data);
	}
}

private void savePersistedData()
{	
	for(int loop=0;loop<persist.size();loop++)
	{
		XmlElement entry = (XmlElement)persist.elementAt(loop);

		FileHelper.buildFile(entry.getAttributeValueByName("in"),getFormData(entry.getAttributeValueByName("from")).toXml());
	}
	
	XappPlugin[] plugins = pluginStore.getPlugins();
	for(int loop=0;loop<plugins.length;loop++)
	{
		XmlElement data = plugins[loop].getData();
		if(data!=null)
		{
			FileHelper.buildFile(plugins[loop].getPersistanceFile(),data.toXml());
		}
	}
}

public void setContainer(String name,XappContainer container)
{
	componentStore.addXappComponent(name,container);
	
	if(container instanceof XappPersister && persist!=null && persist.contains(container)==false)
	{
		persist.add(container);
	}
}

public XappComponent getComponent(String containerName,String componentName)
{
	return componentStore.getXappComponent(containerName+"."+componentName);
}

public XmlElement getFormData(String formName)
{
	return ((XappPersister)componentStore.getXappComponent(formName)).getData();
}

public void setData(String formName,XmlElement data)
{
	((XappPersister)componentStore.getXappComponent(formName)).setData( data );
}

public Hashtable getDataAsHashtable(String formName)
{
	XmlElement data = getFormData(formName);
	Hashtable hash = new Hashtable();
	for(int loop=0;loop<data.getElementCount();loop++)
	{
		hash.put(data.getElementByIndex(loop).getName(),data.getElementByIndex(loop).getValue());
	}
	
	return hash;
}

void _displayException(Exception e)
{
	if(e instanceof InvocationTargetException)
	{
		Throwable t = ((InvocationTargetException)e).getTargetException();
		if(t instanceof Exception)
		{
			e = (Exception)t;
		}
		else
		{
			t.printStackTrace();
			return;
		}
	}

	String message = null;
	String exceptionMessage = e.getMessage();
	if(exceptionMessage!=null)
	{
		message = XappDB.findValue( exceptionMessage );
	}
	if(message!=null && !message.equals(e.getMessage()))
	{
		try{
			e = (Exception)e.getClass().getConstructor(new Class[]{String.class}).newInstance(new Object[]{message});
		}
		catch(Exception failed){}
	}
	
	String[] stack = ExceptionHelper.getStackTrace(e);
	frame.showError( StringArrayHelper.arrayToString(stack,"\n") );
}

void _displayMessage(String message)
{
	message = XappDB.findValue(message);
	frame.showMessage(message);
}

private void buildView(XmlElement xView)
{
	Vector forms = xView.getElementsByName("form");
	for(int loop=0;loop<forms.size();loop++)
	{
		XmlElement xForm = (XmlElement)forms.elementAt(loop);
		
		XForm form = XappRootApplication.getComponentFactory().buildForm( xForm,componentStore );
		componentStore.addXappComponent( form.getName(),form );
	}
	
	XmlElement xContainer = xView.getElementByName("container");
	buildContainer( xContainer );
}

private void buildContainer(XmlElement xContainer)
{
	
	componentFactory.buildContainers(xContainer,componentStore);
	XappContainer container = (XappContainer)componentStore.getXappComponent("root");
	frame.setContainer(container);

	frame.setForegroundColor( container.getForeground() );
	frame.setBackgroundColor( container.getBackground() );

	container.refresh();
}

private void buildController(XmlElement controller)
{		
	Vector receive = controller.getElementsByName("recieveXappData");
	for(int loop=0;loop<receive.size();loop++)
	{
		XmlElement xSource = (XmlElement)receive.elementAt(loop);
		((XappSender)componentStore.getXappComponent(xSource.getAttributeValueByName("from"))).addReceiver(this);
	}		
	
//TODO
/*	persist = controller.getElementsByName("persistXappData");
	if(persist.size()!=0)
	{
		retrievePersistedData();
	}*/
	
/*	pluginStore = new XappPluginStore();
	Vector pluginFolders = controller.getElementsByName("pluginFolder");
	for(int loop=0;loop<pluginFolders.size();loop++)
	{
		XmlElement xPluginFolder = (XmlElement)pluginFolders.elementAt(loop);
		loadPlugins( xPluginFolder.getValue(),controller );
	}*/
	
	Vector drag = controller.getElementsByName("activateDrag");
	for(int loop=0;loop<drag.size();loop++)
	{
		XmlElement xTarget = (XmlElement)drag.elementAt(loop);
		XappDragSource dragSource = componentFactory.getDragAndDropFactory().buildXappDragSource(XappRootApplication.getComponent(xTarget.getAttributeValueByName("from")),(XappPersister)XappRootApplication.getComponent(xTarget.getAttributeValueByName("from")),xTarget.getAttributeValueByName("as"));

		componentStore.addDragSource(xTarget.getAttributeValueByName("as"),dragSource);
	}	
	
	Vector drop = controller.getElementsByName("activateDrop");
	for(int loop=0;loop<drop.size();loop++)
	{
		XmlElement xTarget = (XmlElement)drop.elementAt(loop);
		XappDropReceiver dropReceiver = componentFactory.getDragAndDropFactory().buildXappDropReceiver(componentStore.getXappComponent(xTarget.getAttributeValueByName("on")),this,xTarget.getAttributeValueByName("as"),xTarget.getAttributeValueByName("action"));
		componentStore.addDropTarget(xTarget.getAttributeValueByName("as"),dropReceiver);
	}
}

private void loadPlugins(String folderName,XmlElement controller)
{
	File folder = new File(folderName).getAbsoluteFile();
	if( !folder.isDirectory() )
	{
		return;
	}
	
	File[] list = folder.listFiles();
	for(int loop=0;loop<list.length;loop++)
	{
		if( FileHelper.getFileExtension(list[loop].getName()).equals("plugin") )
		{
			loadPlugin(list[loop],controller);
		}
	}
}

private void initialisePlugins()
{
	XappPlugin[] plugins = pluginStore.getPlugins();
	for(int loop=0;loop<plugins.length;loop++)
	{
		plugins[loop].initialise();
	}
}

public void removePlugin(XappPlugin plugin,XmlElement controller)
{
	XmlElement menu = plugin.getMenu();
	if(menu!=null)
	{
		getXappMenuBar().removeMenu( menu );
	}

	XmlElement actions = plugin.getActions();
	if(actions!=null)
	{
		for(int loopA=0;loopA<actions.getElementCount();loopA++)
		{
			controller.removeChild(actions.getElementByIndex(loopA));
		}
	}
	
	pluginStore.removePlugin(plugin.getAlias());
}

public XappPlugin loadPlugin(File pluginFile,XmlElement controller)
{
	XappPlugin plugin = XappPluginFactory.buildPlugin( pluginFile );
	
	if(pluginStore.getPlugin(plugin.getAlias())!=null)
	{
		_displayMessage("A plugin with the name "+plugin.getAlias()+" is already loaded.");
		return null;
	}
	
	XmlElement menu = plugin.getMenu();
	if(menu!=null)
	{
		getXappMenuBar().addMenu( menu );
	}

	XmlElement actions = plugin.getActions();
	if(actions!=null)
	{
		for(int loopA=0;loopA<actions.getElementCount();loopA++)
		{
			controller.addChild(actions.getElementByIndex(loopA));
		}
	}
	
	plugin.addReceiver( this );
	pluginStore.addPlugin( plugin.getAlias(),plugin );
	
	if( new File(plugin.getPersistanceFile()).exists())
	{
		XmlElement data = XmlLoader.load( plugin.getPersistanceFile() );
	
		if(data!=null)
		{
			plugin.setData( data );
		}
	}
	
	//plugin.initialise();
	
	return plugin;
}

public XappMenuBar getXappMenuBar()
{
	return (XappMenuBar)frame.getXappMenuBar();
}

public void addReceiver(XappReceiver receiver) {
}

public void removeReceiver(XappReceiver receiver) {
}

public void queuing()
{
}

public XappApplicationFrame getFrame() {
	return frame;
}

public String toString()
{
	StringBuffer buffer = new StringBuffer();
	for(int loop=0;loop<transactionQueue.size();loop++)
	{
		buffer.append( ((XmlElement)transactionQueue.elementAt(loop)).toXml() );
	}
	
	return buffer.toString();
}

public void dbChanged(XappXPathDB xPathDB) {
	reinitialise();
}

public void setRecorder(ScreenCapture recorder)
{
	frame.setRecorder(recorder);
}

public boolean isRootApplication() {
	return rootApplication;
}

public void setRootApplication(boolean rootApplication) {
	this.rootApplication = rootApplication;
}

public XappXPathDB getXAppConfig() {
	return xAppConfig;
}

public boolean isAutoSaveOn() {
	return autoSave;
}

public void setAutoSave(boolean autoSave) {
	this.autoSave = autoSave;
}

public XappComponentStore getComponentStore() {
	return componentStore;
}

public XappPluginStore getPluginStore() {
	return pluginStore;
}

public XappComponentFactory getComponentFactory() {
	return componentFactory;
}

}
