package org.one.stone.soup.xapp.components.spreadsheet;

import java.util.Vector;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.one.stone.soup.cursor.TableCursor;
import org.one.stone.soup.xapp.XappDB;
import org.one.stone.soup.xapp.javascript.JavascriptEngine;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlCloner;

public class XappSpreadsheetModel implements TableModel
{
	private Vector listeners=new Vector();
	private XmlElement source;
	private XmlElement data;
	
	private XappSpreadsheet parent;
	private XappSpreadSheetAPI api;
	
	private JavascriptEngine engine;
	
	public XappSpreadsheetModel(XmlElement data,XappSpreadsheet parent)
	{
		this.parent = parent;
		this.source = data;
		this.data = XmlCloner.getClone(data);
		initializeJavascript();
	}
	
	public XmlElement getData()
	{
		return getSource();
	}
	
	private XmlElement getSource()
	{
		XmlElement xSource = new XmlElement("spreadsheet");
		xSource.addAttribute("name",source.getAttributeValueByName("name"));
		xSource.addChild(source.getElementByName("columns"));
		for(int loopR=0;loopR<getRowCount();loopR++)
		{
			XmlElement xRow = xSource.addChild("row");
			for(int loopF=0;loopF<(getColumnCount()-1);loopF++)
			{
				XmlElement xField = xRow.addChild("field"); 
				String field = getSourceField(loopR,loopF).getValue();
				if(field.length()==0)
				{
					field = getField(loopR,loopF).getValue();
				}
				xField.addValue(field);
			}
		}
		return xSource;
	}
	
	public XappSpreadSheetAPI getAPI()
	{
		return api;
	}
	
	public void addTableModelListener(TableModelListener l) {
		listeners.addElement(l);
	}

	public Class getColumnClass(int columnIndex) {
		return String.class;
	}

	public int getColumnCount() {
		return getNumberOfColumns();
	}

	public String getColumnName(int columnIndex) {
		if(columnIndex==0)
		{
			return "";
		}
		return getColumnTitle(columnIndex-1);
	}	
	
	public int getRowCount() {
		return getNumberOfRows();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		if(columnIndex==0)
		{
			return ""+(rowIndex+1);
		}
		else
		{
			columnIndex--;
		}
		
		XmlElement field = getField(rowIndex,columnIndex);
		if( field!=null )
		{
			return field.getValue();
		}
		else
		{
			return null;
		}
	}

	public XappSpreadsheet getParent()
	{
		return parent;
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		XmlElement field = getField(rowIndex,columnIndex-1);
		if( field!=null )
		{
			String editable = field.getAttributeValueByName("enabled");
			if(editable==null || editable.equals("true"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			return false;
		}
	}

	public void removeTableModelListener(TableModelListener l) {
		listeners.remove(l);
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if(columnIndex==0)
		{
			return;
		}
		else
		{
			columnIndex--;
		}
		
		XmlElement field = getField(rowIndex,columnIndex);
		if(field!=null)
		{
			field.setValue(aValue.toString(),0);
		}
		field = getSourceField(rowIndex,columnIndex);
		if(field!=null)
		{
			field.setValue(aValue.toString(),0);			
		}
	}
	
	private int getNumberOfRows()
	{
		return data.getElementsByName("row").size();
	}
	
	private int getNumberOfColumns()
	{
		XmlElement row = data.getElementByName("columns");
		if(row==null)
		{
			row = data.getElementByName("row");
		}
		if(row==null)
		{
			return 1;
		}
		else
		{
			return row.getElementCount()+1;
		}
	}
	
	private String getColumnTitle(int columnIndex)
	{
		XmlElement row = data.getElementByName("columns");
		if(row==null)
		{
			return null;
		}
		else
		{
			if(row.getElementCount()<=columnIndex)
			{
				return null;
			}
			else
			{
				return row.getElementByIndex(columnIndex).getValue();
			}
		}
	}
	
	public XmlElement getColumnRenderer(int columnIndex)
	{
		if(columnIndex==0)
		{
			return null;
		}		
		columnIndex--;
		
		XmlElement row = data.getElementByName("columns");
		if(row==null)
		{
			return null;
		}
		else
		{
			if(row.getElementCount()<=columnIndex)
			{
				return null;
			}
			else
			{
				return row.getElementByIndex(columnIndex).getElementByName("renderer");
			}
		}
	}	
	
	public int getColumnIndex(String name)
	{
		XmlElement row = data.getElementByName("columns");
		if(row==null)
		{
			return -1;
		}
		for(int loop=0;loop<row.getElementCount();loop++)
		{
			if(row.getElementByIndex(loop).getValue().equals(name))
			{
				return loop;
			}
		}
		return -1;
	}

	public XmlElement getSourceField(int rowIndex,int columnIndex)
	{
		Vector rows = source.getElementsByName("row");
		if(rows.size()<=rowIndex)
		{
			return null;
		}
		else
		{
			XmlElement row = (XmlElement)rows.elementAt(rowIndex);
			if(row.getElementCount()<=columnIndex || columnIndex<0)
			{
				return null;
			}
			else
			{
				return row.getElementByIndex(columnIndex);
			}
		}		
	}
	
	public XmlElement getField(int rowIndex,int columnIndex)
	{
		Vector rows = data.getElementsByName("row");
		if(rows.size()<=rowIndex)
		{
			return null;
		}
		else
		{
			XmlElement row = (XmlElement)rows.elementAt(rowIndex);
			if(row.getElementCount()<=columnIndex || columnIndex<0)
			{
				return null;
			}
			else
			{
				return row.getElementByIndex(columnIndex);
			}
		}		
	}
	
	public String getValueAt(String cellName)
	{
		TableCursor cursor = getTableCursor(cellName);
		if(cursor.row<0 || cursor.field<0)
		{
			return null;
		}
		
		XmlElement field = getField(cursor.row,cursor.field);
		if(field==null)
		{
			return null;
		}
		
		return field.getValue();
	}

	public String getSourceField(String cellName)
	{
		TableCursor cursor = getTableCursor(cellName);
		
		return getSourceField(cursor.row,cursor.field).getValue();
	}
	
	public void setField(String cellName,String value)
	{
		TableCursor cursor = getTableCursor(cellName);
		
		getField(cursor.row,cursor.field).setValue(value,0);
		parent.refresh();
	}

	public void setField(int row,int column,String value)
	{
		getField(row,column).setValue(value,0);
		parent.refresh();
	}
	
	public void setSourceField(String cellName,String value)
	{
		TableCursor cursor = getTableCursor(cellName);
		
		getSourceField(cursor.row,cursor.field).setValue(value,0);
		parent.refresh();
	}
	
	public TableCursor getTableCursor(String cellName)
	{
		int cursor = 0;
		while( cursor<cellName.length() && !Character.isDigit(cellName.charAt(cursor)) )
		{
			cursor++;
		}
		if(cursor==cellName.length())
		{
			return new TableCursor(-1,-1);
		}
		
		String columnName = cellName.substring(0,cursor);
		int column = getColumnIndex(columnName);
		int row = Integer.parseInt(cellName.substring(cursor))-1;
		
		return new TableCursor(column,row);
	}	
	
	private void initializeJavascript()
	{
		api = new XappSpreadSheetAPI(this);
		
		engine = new JavascriptEngine();
		try{
			engine.register(api,"Spreadsheet");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public void processFieldFunction(int rowIndex,int columnIndex)
	{
		if(columnIndex==0)
		{
			return;
		}
		columnIndex--;
		XmlElement script = getSourceField(rowIndex,columnIndex);
		XmlElement field = getField(rowIndex,columnIndex);
		if(script==null || script.getValue()==null || script.getValue().length()==0)
		{
			return;
		}

		api.cursor.row = rowIndex;
		api.cursor.field = columnIndex;

		String value = XappDB.findValue( script.getValue(),engine );
		field.setValue( value,0 );
	}
}
