package org.one.stone.soup.xapp.controller.xml;

import java.util.Enumeration;
import java.util.Hashtable;

import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xapp.XappComponentStore;
import org.one.stone.soup.xapp.XappDB;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.components.form.XFormAdapter;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlCloner;

public class XmlFormFactory {
	private XmlSchemaController schema;
	private XmlSchemaViewController view;
	
	//private XappSubXPathDB xViewDB;
	private XmlApplicationController application;
	
	public XmlFormFactory(XmlApplicationController application,XmlSchemaController schema,XmlSchemaViewController view)
	{
		this.application = application;
		this.schema = schema;
		this.view = view;
	}
	
	public XForm createForm(XmlElement root,XmlElement node,XappComponentStore components)
	{
		XmlElement xForm = view.getFormForNode(node);
		if(xForm!=null)
		{
			String formName = xForm.getAttributeValueByName("name");
			String adapter = xForm.getAttributeValueByName("adapter");	
			String producer = xForm.getAttributeValueByName("producer");			
			
			if(formName.charAt(0)=='#')
			{
				xForm = XappRootApplication.getFormDefinition(formName.substring(1));
				
				if(adapter==null)
				{
					adapter = xForm.getAttributeValueByName("adapter");					
				}
				
				if(producer==null)
				{
					producer = xForm.getAttributeValueByName("producer");					
				}
			}
			else
			{
				xForm = XmlCloner.getClone(xForm);
			}
			
			if(producer!=null)
			{
				try{
					xForm = ((XFormProducer)Class.forName(producer).newInstance() ).buildFormFor(this,root,node);
				}
				catch(Exception e)
				{
					XappRootApplication.displayException(e);
				}				
			}
			
			xForm.addAttribute("name","xmlNodeForm");
			xForm.addAttribute("cellPadding","10");
			xForm.addAttribute("rowPadding","2");
			xForm.addAttribute("realtime","true");
			
			XForm form = XappRootApplication.getComponentFactory().buildForm(xForm,components);
			if(adapter!=null)
			{
				try{
					form.setAdapter( (XFormAdapter)Class.forName(adapter).newInstance() );
				}
				catch(Exception e)
				{
					XappRootApplication.displayException(e);
				}
			}
			else
			{
				form.setAdapter(new XmlFormAdapter());
			}
			
			form.setData(node);
			
			return form;
		}
			
		xForm = new XmlElement("table");
		xForm.addAttribute("name","xmlNodeForm");
		xForm.addAttribute("cellPadding","10");
		xForm.addAttribute("rowPadding","1");
		xForm.addAttribute("realtime","true");
		
		xForm.addChild("tr").addChild("td").addChild("h").addValue(node.getName());

		
		for(int loop=0;loop<node.getAttributeCount();loop++)
		{
			String attributeName = node.getAttributeNameByIndex(loop);
			if(attributeName.charAt(0)=='_')
			{
				buildSystemAttributeEntryRow( xForm,node,attributeName,node.getAttributeValueByIndex(loop));
			}
			else
			{
				xForm.addChild( buildAttributeEntryRow( node.getName(),attributeName,node.getAttributeValueByIndex(loop)) );
			}
		}

		for(int loop=0;loop<node.getElementCount();loop++)
		{
			if(node.getElementByIndex(loop)!=null)
			{
				xForm.addChild( buildElementViewRow( node.getElementByIndex(loop)) );
			}
		}		
		
		if(schema.canHaveCData(node))
		{
			StringBuffer cData = new StringBuffer();
			for(int loop=0;loop<node.getElementCount();loop++)
			{
				if(node.getElementByIndex(loop)==null)
				{
					cData.append( node.getValue(loop) );
				}
			}
			xForm.addChild( buildCDataEntryRow(node.getName(),cData.toString()) );		
		}
		
		XForm form = XappRootApplication.getComponentFactory().buildForm(xForm,components);
		
		return form;
	}
	
	public XmlElement buildElementViewRow( XmlElement node )
	{
		XmlElement row = new XmlElement("tr");
		
		XmlElement xIcon = row.addChild("td").addChild("img");
		try{
			xIcon.addAttribute("src",view.getIconFileNameForNode(node,false));
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		row.addChild("td").addValue(view.getNameForNode(node));
		
		return row;
	}	
	
	public XmlElement buildAttributeEntryRow(String nodeName,String name,String value)
	{
		XmlElement row = new XmlElement("tr");
		
		row.addChild("td").addValue(name);
		
		XmlElement cell = row.addChild("td");
		if(value==null)
		{
			cell.addValue("not present");
			value="";
		}
		
		XmlElement input = cell.addChild("input");
		
		String type = schema.getAttributeTypeFor(nodeName,name);

		if(type.toLowerCase().equals("boolean"))
		{
			input.addAttribute("type","checkbox");
			input.addAttribute("name",name);
			input.addAttribute("value",value);
		}
		else if(type.toLowerCase().equals("color"))
		{
			input.addAttribute("type","color");
			input.addAttribute("name",name);
			input.addAttribute("value",value);
		}
		else if(type.toLowerCase().equals("file"))
		{
			input.addAttribute("type","file");
			input.addAttribute("name",name);
			input.addAttribute("value",value);
		}
		else
		{	
			input.addAttribute("type","text");
			input.addAttribute("name",name);
			input.addAttribute("size",""+(value.length()+4));
			input.addAttribute("value",value);
			if(type!=null)
			{
				if(type.length()>5 && type.substring(0,5).equals("list:"))
				{
					String listName = type.substring(5);
					XmlElement xList = schema.getList(listName);
					
					input.setName("select");
					input.addAttribute("name",name);
					input.addAttribute("value",value);
					
					if(xList==null)
					{
						input.addAttribute("editable","true");
						input.addChild("option").addValue("");
					}
					else
					{
						for(int loopX=0;loopX<xList.getElementCount();loopX++)
						{
							XmlElement xEntry = xList.getElementByIndex(loopX);
							if(xEntry!=null)
							{
								continue;
							}
							String list = xList.getValue(loopX); 
							String[] options = StringArrayHelper.parseFields(list,'\n');
						
							for(int loop=0;loop<options.length;loop++)
							{
								String data = options[loop].trim();
								if(data.indexOf('*')!=-1 && input.getAttributeValueByName("editable")==null)
								{
									input.addAttribute("editable","true");
								}
								
								input.addChild("option").addValue(data);
							}
						}
					}
				}
				else if(type.indexOf(':')>0)
				{
					/*String searchName = type.substring(6);
					int pos = searchName.lastIndexOf("@");
					String attributeName = null;
					if(pos>0)
					{
						attributeName = searchName.substring(pos+1);
						if(attributeName.indexOf(']')!=-1)
						{
							attributeName = null;
						}
						else
						{
							searchName = searchName.substring(0,pos); 
						}
					}
					
					XmlElement result = new XmlPathSearch(application.getModelRoot()).findElements( searchName );*/
					
					Hashtable list = XappDB.findList(type);
					
					input.setName("select");
					input.addAttribute("name",name);
					input.addAttribute("value",value);
					
					Enumeration enumeration = list.keys();
					
					while(enumeration.hasMoreElements())
					{
						String key = (String)enumeration.nextElement();
						String keyValue = (String)list.get(key);
						XmlElement option = new XmlElement("option");
						option.addValue(keyValue);
						option.addAttribute("value",key);
						input.addChild(option);
					}
					
				}
					
				input.addAttribute("format",type.toLowerCase());
			}
		}
		
		return row; 
	}
	
	public void buildSystemAttributeEntryRow(XmlElement form,XmlElement node,String name,String value)
	{
		String viewName = name.substring(1);
		if(name.equals("_src"))
		{
			viewName="Source File";
			XmlElement row = new XmlElement("tr");
			row.addChild("td").addValue(viewName);
			XmlElement cell = row.addChild("td");
			if(value==null)
			{
				cell.addValue("not present");
				value="";
			}
			else
			{
				cell.addValue(value);
			}
			
			form.addChild(row);
			
			row = new XmlElement("tr");
			row.addChild("td").addValue("Line numbers");
			cell = row.addChild("td");
			cell.addValue( node.getStartLineNo()+" to "+node.getEndLineNo() );
			
			form.addChild(row);
		}
	}	
	
	public XmlElement buildCDataEntryRow(String nodeName,String value)
	{
		XmlElement row = new XmlElement("tr");
		
		row.addChild("td").addValue("Value:");
		
		XmlElement cell = row.addChild("td");
		XmlElement input = cell.addChild("input");
		
		String type = schema.getCDataTypeFor(nodeName);
		if(type==null || type.length()==0)
		{
			type="string";
		}

		if(type.toLowerCase().equals("boolean"))
		{
			input.addAttribute("type","checkbox");
			input.addAttribute("name","@value");
			input.addAttribute("value",value);
		}
		else if(type.toLowerCase().equals("color"))
		{
			input.addAttribute("type","color");
			input.addAttribute("name","@value");
			input.addAttribute("value",value);
		}
		else if(type.toLowerCase().equals("file"))
		{
			input.addAttribute("type","file");
			input.addAttribute("name","@value");
			input.addAttribute("value",value);
		}
		else
		{	
			input.addAttribute("type","text");
			input.addAttribute("name","@value");
			input.addAttribute("value",value);
			if(type!=null)
			{
				if(type.toLowerCase().equals("string"))
				{
					input.addAttribute("name","@value");
					input.addAttribute("size",""+(value.trim().length()+4));

					input.addValue(value.trim());					
				}
				else if(type.toLowerCase().equals("text"))
				{
					input.setName("textarea");
					input.addAttribute("name","@value");
					input.addAttribute("style","normal");
					
					int rows = StringArrayHelper.parseFields(value.trim(),'\n').length+4;
					input.addAttribute("rows",""+rows);
					input.addAttribute("cols","40");

					input.addValue(value.trim());					
				}
				else
				{
					if(type.length()>5 && type.substring(0,5).equals("list:"))
					{
						String listName = type.substring(5);
						XmlElement xList = schema.getList(listName);
						
						input.setName("select");
						input.addAttribute("name","@value");
						input.addAttribute("value",value);
						
						if(xList==null)
						{
							input.addAttribute("editable","true");
						}
						else
						{
							for(int loopX=0;loopX<xList.getElementCount();loopX++)
							{
								XmlElement xEntry = xList.getElementByIndex(loopX);
								if(xEntry!=null)
								{
									continue;
								}
								String list = xList.getValue(loopX); 
								String[] options = StringArrayHelper.parseFields(list,'\n');
							
								for(int loop=0;loop<options.length;loop++)
								{
									String data = options[loop].trim();
									if(data.indexOf('*')!=-1 && input.getAttributeValueByName("editable")==null)
									{
										input.addAttribute("editable","true");
									}
									input.addChild("option").addValue(data);
								}
							}
						}
					}
						
					input.addAttribute("format",type.toLowerCase());
				}
			}
		}
		
		return row; 
	}	
}
