package org.one.stone.soup.xapp.helper;

import java.io.File;

import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.filebrowser.XappFileBrowser;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.XmlParser;
import org.one.stone.soup.xml.stream.XmlLoader;

public class XappWorkingFileManager {

	private String workingFileName;
	private String defaultFileName;
	private String fileTypeName;
	private String fileExtension;
	
	public XappWorkingFileManager(String workingFileName,String defaultFileName,String fileTypeName,String fileExtension)
	{
		this.workingFileName=workingFileName;
		this.defaultFileName=defaultFileName;
		this.fileTypeName=fileTypeName;
		this.fileExtension=fileExtension;	
	}

	public String getCurrentFile()
	{
		return workingFileName;
	}
	
	public String getFileToOpen()
	{
		XappFileBrowser browser = XappRootApplication.getComponentFactory().buildFileBrowser(" "+fileTypeName,fileTypeName,fileExtension);
		if(workingFileName!=null)
		{
			browser.setOpenDir( new File(workingFileName).getParent() );
		}
		String file = browser.getOpenFile();
		
		if(file!=null)
		{
			workingFileName = file;
			XappRootApplication.getRoot().getFrame().setTitle( XappRootApplication.getRoot().getFrame().getTitle()+" - "+new File(file).getName() );
			return workingFileName;
		}
		return null;
	}
	
	public String getFileToSaveTo()
	{
		if(workingFileName==null || workingFileName.equals("*"))
		{
			return saveAs();
		}
		else
		{
			return workingFileName;
		}		
	}

	public String saveAs()
	{
		XappFileBrowser browser = XappRootApplication.getComponentFactory().buildFileBrowser(" "+fileTypeName,fileTypeName,fileExtension);
		
		String file=null;
		
		if(workingFileName==null)
		{
			file = browser.getSaveAsFile( defaultFileName );			
		}
		else
		{
			file = browser.getSaveAsFile( new File(workingFileName).getName() );
		}
		
		if(file!=null)
		{
			workingFileName = file;
			XappRootApplication.getRoot().getFrame().setTitle( XappRootApplication.getRoot().getFrame().getTitle()+" - "+new File(file).getName() );
			return workingFileName;
		}
		else
		{
			return null;
		}
	}

	public String openFileAsString()
	{
		String fileName = getFileToOpen();
		
		if(fileName!=null)
		{
			try{
				return FileHelper.readFile( fileName );
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
				return null;
			}
		}
		else
		{
			return null;
		}
	}
	
	public XmlElement openFileAsXml()
	{
		return openFileAsXml(XmlLoader.getStandardParser());
	}
	
	public XmlElement openFileAsXml(XmlParser parser)
	{
		String fileName = getFileToOpen();
		
		if(fileName!=null)
		{
			try{
				return XmlLoader.load( fileName,parser );
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
				return null;
			}
		}
		else
		{
			return null;
		}
	}
	
	public boolean save(String data)
	{
		String fileName = getFileToSaveTo();
		
		if(fileName!=null)
		{
			FileHelper.buildFile(fileName,data);
			return true;
		}
		return false;
	}
	
	public boolean save(XmlElement data)
	{
		String fileName = getFileToSaveTo();
		
		if(fileName!=null)
		{
			FileHelper.buildFile(fileName,data.toXml());
			return true;
		}
		return false;
	}
	
	public void clearWorkingFileName()
	{
		workingFileName=null;
		XappRootApplication.getRoot().getFrame().setTitle( XappRootApplication.getRoot().getFrame().getTitle()+" - * Untitled" );		
	}
	
	public void setWorkingFileName(String fileName)
	{
		workingFileName = fileName;
		XappRootApplication.getRoot().getFrame().setTitle( XappRootApplication.getRoot().getFrame().getTitle()+" - "+new File(workingFileName).getName() );
	}
	
	public boolean isWorkingFileSet()
	{
		if(workingFileName==null)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
}
