package org.one.stone.soup.xapp.swing.components;

import java.awt.*;
import java.util.*;

import javax.swing.JInternalFrame;

import org.one.stone.soup.xapp.XappComponentStore;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.form.XFormTableHelper;
import org.one.stone.soup.xapp.containers.XappContainer;
import org.one.stone.soup.xml.*;

public class XSwingFormLayout implements LayoutManager2 {

	private String name;

	private static final int ALIGN_LEFT=1;
	private static final int ALIGN_CENTER=2;
	private static final int ALIGN_RIGHT=3;

	private static final int ALIGN_TOP=4;
	private static final int ALIGN_MIDDLE=5;
	private static final int ALIGN_BOTTOM=6;

	private int[] rowSize;
	private int[] colSize;

	private int[][] cellAlign;
	private int[][] cellVAlign;
	private int[][] cellColSpan;
	private int[][] cellRowSpan;
	private Color[][] cellBgColor;
	private String[][] cellName;

	private Hashtable cells = new Hashtable();

	private boolean flexible = true;
/**
 * TableLayout constructor comment.
 */
public XSwingFormLayout(XmlElement table) {
	super();

	buildTable(table);
}
	/**
	 * Adds the specified component to the layout, using the specified
	 * constraint object.
	 * @param comp the component to be added
	 * @param constraints  where/how the component is added to the layout.
	 */
public void addLayoutComponent(Component comp, Object constraints)
{
	if(constraints!=null)
	{
		cells.put(constraints,comp);
	}
}
public void addLayoutComponent(String name, Component comp)
{
	// add component to cell with name
	cells.put(name,comp);
}
private void buildTable(XmlElement table)
{
	String name = table.getAttributeValueByName("name");
	if(name!=null)
	{
		setName(name);
	}

	Vector xRows = table.getElementsByName("tr");

	// find row count
	int rows = xRows.size();
	int cols=1;

	// find largest columns count
	for(int loopR=0;loopR<rows;loopR++)
	{
		XmlElement xRow = (XmlElement)xRows.elementAt(loopR);
		Vector xCols = xRow.getElementsByName("td");

		if(xCols.size()>cols)
		{
			cols = xCols.size();
		}
	}

	rowSize = new int[rows];
	colSize = new int[cols];

	cellColSpan = new int[rows][cols];
	cellRowSpan = new int[rows][cols];
	cellAlign = new int[rows][cols];
	cellVAlign = new int[rows][cols];
	cellName = new String[rows][cols];
	cellBgColor = new Color[rows][cols];

//	int rowOffset = 0;

	for(int loopR=0;loopR<xRows.size();loopR++)
	{
		// find row height
		XmlElement xRow = (XmlElement)xRows.elementAt(loopR);
		String rowHeight = xRow.getAttributeValueByName("height");
		if( rowHeight!=null && rowHeight.length()>0 && Integer.parseInt(rowHeight)>rowSize[loopR] )
		{
			rowSize[loopR]=Integer.parseInt(rowHeight);
		}

		Vector xCols = xRow.getElementsByName("td");

//		int colOffset = 0;

		for(int loopC=0;loopC<xCols.size();loopC++)
		{
			XmlElement xCol = (XmlElement)xCols.elementAt(loopC);


			String align = xCol.getAttributeValueByName("align");
			String valign = xCol.getAttributeValueByName("valign");
			String colspan = xCol.getAttributeValueByName("colspan");
			String rowspan = xCol.getAttributeValueByName("rowspan");
			String bgColor = xCol.getAttributeValueByName("bgcolor");

			// set cell background color
			if(bgColor!=null)
			{
				cellBgColor[loopR][loopC] = org.one.stone.soup.grfx.ColorHelper.htmlToColor(bgColor);
			}

			String cellHeight = xCol.getAttributeValueByName("height");
			String cellWidth = xCol.getAttributeValueByName("width");

			// find maximum row height
			if(rowspan==null && cellHeight!=null && cellHeight.length()>0 && Integer.parseInt(cellHeight)>rowSize[loopR] )
			{
				rowSize[loopR]=Integer.parseInt(cellHeight);
			}

			// find maximum col height
			if(colspan==null && cellWidth!=null && cellWidth.length()>0 && Integer.parseInt(cellWidth)>colSize[loopC] )
			{
				colSize[loopC]=Integer.parseInt(cellWidth);
			}

			name = xCol.getAttributeValueByName("name");

			// create a cell name if no name exists
			if(name==null || name.length()==0)
			{
				name="r"+loopR+"c"+loopC;
			}

			// set the column span
			if(colspan!=null && colspan.length()>0)
			{
				cellColSpan[loopR][loopC] = Integer.parseInt(colspan);
			}
			else
			{
				cellColSpan[loopR][loopC]=1;
			}

			// set the row span
			if(rowspan!=null)
			{
				cellRowSpan[loopR][loopC] = Integer.parseInt(rowspan);
			}
			else
			{
				cellRowSpan[loopR][loopC]=1;
			}

			// set cell alignment
			cellAlign[loopR][loopC]=getAlign(align);
			cellVAlign[loopR][loopC]=getAlign(valign);

			// set cell name
			cellName[loopR][loopC]=name;
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public boolean cellEmpty(int row, int col) {
	if(cells.get(cellName[row][col])==null)
	{
		return true;
	}
	else
	{
		return false;
	}
}
private int getAlign(String key) {

	if(key==null)
	{
		return 0;
	}

	key=key.toLowerCase();

	if(key.equals("left"))
	{
		return ALIGN_LEFT;
	}
	else if(key.equals("center"))
	{
		return ALIGN_CENTER;
	}
	else if(key.equals("right"))
	{
		return ALIGN_RIGHT;
	}
	else if(key.equals("top"))
	{
		return ALIGN_TOP;
	}
	else if(key.equals("middle"))
	{
		return ALIGN_MIDDLE;
	}
	else if(key.equals("bottom"))
	{
		return ALIGN_BOTTOM;
	}

	return 0;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public Color getCellBgColor(int row, int col) {
	return cellBgColor[row][col];
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getCellHeight(int row, int col) {

	int height=0;

	for(int loop=0;loop<cellRowSpan[row][col];loop++)
	{
		height+=rowSize[row+loop];
	}

	return height;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getCellWidth(int row, int col) {

	int width=0;

	for(int loop=0;loop<cellColSpan[row][col];loop++)
	{
		if(col+loop>=colSize.length)
		{
			break;
		}
		width+=colSize[col+loop];
	}

	return width;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getColumnCount() {

	return colSize.length;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getColumnSize(int col) {

	return colSize[col];
}
/**
 * Insert the method's description here.
 * Creation date: (07/05/03 16:35:15)
 * @return java.awt.Component
 * @param name java.lang.String
 */
public Component getComponent(String name) {
	return (Component)cells.get(name);
}
	/**
	 * Returns the alignment along the x axis.  This specifies how
	 * the component would like to be aligned relative to other
	 * components.  The value should be a number between 0 and 1
	 * where 0 represents alignment along the origin, 1 is aligned
	 * the furthest away from the origin, 0.5 is centered, etc.
	 */
public float getLayoutAlignmentX(Container target) {
	return 0;
}
	/**
	 * Returns the alignment along the y axis.  This specifies how
	 * the component would like to be aligned relative to other
	 * components.  The value should be a number between 0 and 1
	 * where 0 represents alignment along the origin, 1 is aligned
	 * the furthest away from the origin, 0.5 is centered, etc.
	 */
public float getLayoutAlignmentY(Container target) {
	return 0;
}
/**
 * Insert the method's description here.
 * Creation date: (23/04/03 18:32:55)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return name;
}
private int getOffsetX(int row,int col,Component comp)
{
	int align = cellAlign[row][col];
	if(align==ALIGN_LEFT)
	{
		return 0;
	}

	int width = (int)comp.getPreferredSize().getWidth();

	int maxWidth = getCellWidth(row,col);//colSize[col];

	if(align==ALIGN_CENTER)
	{
		if(width>=maxWidth)
		{
			return 0;
		}
		else
		{
			return (maxWidth-width)/2;
		}
	}

	if(align==ALIGN_RIGHT)
	{
		if(width>=maxWidth)
		{
			return 0;
		}
		else
		{
			return maxWidth-width;
		}
	}

	return 0;
}
private int getOffsetY(int row,int col,Component comp)
{
	int valign = cellVAlign[row][col];
	if(valign==ALIGN_TOP)
	{
		return 0;
	}

	int height = (int)comp.getPreferredSize().getHeight();

	int maxHeight = getCellHeight(row,col);//rowSize[row];

	if(valign==ALIGN_MIDDLE)
	{
		if(height>=maxHeight)
		{
			return 0;
		}
		else
		{
			return (maxHeight-height)/2;
		}
	}

	if(valign==ALIGN_BOTTOM)
	{
		if(height>=maxHeight)
		{
			return 0;
		}
		else
		{
			return maxHeight-height;
		}
	}

	return 0;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getRowCount() {

	return rowSize.length;
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:25:34)
 * @return int
 * @param row int
 * @param col int
 */
public int getRowSize(int row) {

	return rowSize[row];
}

public String getCellName(Component component)
{
	Enumeration enumeration = cells.keys();
	String name = null;

	while(enumeration.hasMoreElements())
	{
		String key = (String)enumeration.nextElement();
		if(cells.get(key)==component)
		{
			name = key;
			break;
		}
	}

	return name;
}

public String getCellNameAt(int x,int y)
{
	int startX = 0;
	int startY = 0;

	Dimension cSize = null;
	for(int loopR=0;loopR<rowSize.length;loopR++)
	{
		for(int loopC=0;loopC<colSize.length;loopC++)
		{
			cSize = getCellSize( loopR,loopC );
			if(
				x >= startX && x<= (startX+cSize.width) &&
				y >= startY && y<= (startY+cSize.height)
			)
			{
				return cellName[loopR][loopC];
			}

			startX+=colSize[loopC];
		}
		startX = 0;
		startY+=rowSize[loopR];
	}

	return null;
}

public Dimension getCellSize( int row,int col )
{
	Dimension size = new Dimension();

	int cSpan = cellColSpan[row][col];
	int rSpan = cellRowSpan[row][col];

	for(int loopR=0;loopR<rSpan;loopR++)
	{
		if(row+loopR>=rowSize.length)
		{
			break;
		}
		size.height += rowSize[row+loopR];
	}

	for(int loopC=0;loopC<cSpan;loopC++)
	{
		if(col+loopC>=colSize.length)
		{
			break;
		}
		size.width += colSize[col+loopC];
	}

	return size;
}

public Component getCell(int row,int col)
{
	if(cellName[row][col]==null)
	{
		return null;
	}

	return (Component)cells.get(cellName[row][col]);
}

	/**
	 * Invalidates the layout, indicating that if the layout manager
	 * has cached information it should be discarded.
	 */
public void invalidateLayout(Container target) {}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:09:01)
 * @return boolean
 */
public boolean isFlexible() {
	return flexible;
}
	/**
	 * Lays out the container in the specified panel.
	 * @param parent the component which needs to be laid out
	 */
public void layoutContainer(Container parent)
{
	if(flexible)
	{
		for(int loopR=0;loopR<rowSize.length;loopR++)
		{
			for(int loopC=0;loopC<colSize.length;loopC++)
			{
				if(cellName[loopR][loopC]!=null)
				{
					// find component for cell
					Component comp = (Component)cells.get(cellName[loopR][loopC]);

					if(comp!=null)
					{
						// get component size
						Dimension size = comp.getPreferredSize();

						/*if(comp instanceof XFormTable)
						{
							System.out.println("Table:"+comp.getName()+" size:"+size);
						}*/

						// if the definied cell width is smaller than the component width
						if(colSize[loopC] < size.getWidth())
						{
							// if the cell does not span multiple columns
							if( cellColSpan[loopR][loopC]<2 )
							{
								colSize[loopC] = (int)size.getWidth();
							}
							else
							{
								int width = getCellWidth(loopR,loopC);
								if( width < size.getWidth() )
								{
									int inc = ((int)size.getWidth()-width);

									int loop = (cellColSpan[loopR][loopC]-2)+loopC;
									if(loop>=colSize.length)
									{
										loop = colSize.length-1;
									}
									colSize[loop] += inc;
								}
							}
						}

						if(rowSize[loopR] < size.getHeight())
						{
							if(cellRowSpan[loopR][loopC]<2 )
							{
								rowSize[loopR] = (int)size.getHeight();
							}
							else
							{
								int height = getCellHeight(loopR,loopC);
								if( height < size.getHeight() )
								{
									int inc = ((int)size.getHeight()-height)/(cellRowSpan[loopR][loopC]-1)+1;

									for(int loop=0;loop<(cellRowSpan[loopR][loopC]-1);loop++)
									{
										rowSize[loopC + loop] += inc;
									}
								}
							}
						}
					}
				}
			}
		}
	}

	int x=0;
	int y=0;

	for(int loopR=0;loopR<rowSize.length;loopR++)
	{
		x=0;
		for(int loopC=0;loopC<colSize.length;loopC++)
		{
			if(cellName[loopR][loopC]!=null)
			{
				Component comp = (Component)cells.get(cellName[loopR][loopC]);
				if(comp!=null)
				{
					int offsetX = getOffsetX( loopR,loopC,comp );
					int offsetY = getOffsetY( loopR,loopC,comp );
					int width = getCellWidth( loopR,loopC );
					int height = getCellHeight( loopR,loopC );

					Dimension dim = comp.getPreferredSize();

					if(dim.width<width)
						width = dim.width;
					if(dim.height<height)
						height = dim.height;

					if( comp instanceof JInternalFrame)
					{
						comp.setLocation(x+offsetX,y+offsetY);
						Rectangle rect = comp.getBounds();
						if(rect.width<width || rect.height<height)
						{
							comp.setBounds(x+offsetX,y+offsetY,width,height);
						}
					}
					else
					{
						comp.setBounds(x+offsetX,y+offsetY,width,height);
					}
				}
			}
			x+=colSize[loopC];
		}
		y+=rowSize[loopR];
	}

/*	int x=0;
	int y=0;
	int w=0;
	int h=0;

	for(int loop=0;loop<elements.size();loop++)
	{
		Component comp = (Component)elements.elementAt(loop);

		Dimension size = comp.getPreferredSize();

		w = (int)size.getWidth()+100;
		h = (int)size.getHeight();

		comp.setBounds(x,y,w,h);

		System.out.println("Comp:"+comp+" x:"+x+"y:"+y+"w:"+w+"h:"+h);

		y+=h+10;
	}*/

	//for each element
	//set bounds held by cell
}
	/**
	 * Returns the maximum size of this component.
	 * @see java.awt.Component#getMinimumSize()
	 * @see java.awt.Component#getPreferredSize()
	 * @see LayoutManager
	 */
public Dimension maximumLayoutSize(Container target) {
	return minimumLayoutSize(target);
}
	/**
	 * Calculates the minimum size dimensions for the specified
	 * panel given the components in the specified parent container.
	 * @param parent the component to be laid out
	 * @see #preferredLayoutSize
	 */
public Dimension minimumLayoutSize(java.awt.Container parent) {

	layoutContainer( parent );

	int width = 0;
	for(int loop=0;loop<colSize.length;loop++)
	{
		width+=colSize[loop];
	}

	int height = 0;
	for(int loop=0;loop<rowSize.length;loop++)
	{
		height+=rowSize[loop];
	}

	return new Dimension(width,height);
}
	/**
	 * Calculates the preferred size dimensions for the specified
	 * panel given the components in the specified parent container.
	 * @param parent the component to be laid out
	 *
	 * @see #minimumLayoutSize
	 */
public Dimension preferredLayoutSize(java.awt.Container parent) {
	return minimumLayoutSize(parent);
}
	/**
	 * Removes the specified component from the layout.
	 * @param comp the component to be removed
	 */
public void removeLayoutComponent(Component comp)
{
	String name = comp.getName();
	if(name!=null)
	{
		cells.remove(comp.getName());
	}
}
public void replaceLayoutComponent(Component oldComp,Component newComp,String key)
{
	String name = oldComp.getName();
	if(name!=null)
	{
		cells.remove(oldComp.getName());
		cells.put(key,newComp);
	}
}
/**
 * Insert the method's description here.
 * Creation date: (09/05/03 21:09:01)
 * @param newFlexible boolean
 */
public void setFlexible(boolean newFlexible) {
	flexible = newFlexible;
}
/**
 * Insert the method's description here.
 * Creation date: (23/04/03 18:32:55)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	name = newName;
}

public static XSwingFormLayout buildTableLayout(XSwingForm container,XmlElement table,XappComponentStore componentStore)
{
	//System.out.println("Tbl:"+table.toXml());

	String name = table.getAttributeValueByName("name");
	if(name!=null)
	{
		container.setName(name);
		componentStore.addContainer(name,container);
	}

	String cellPadding = table.getAttributeValueByName("cellPadding");
	String rowPadding = table.getAttributeValueByName("rowPadding");
	if(cellPadding!=null||rowPadding!=null)
	{
		int cellPad = 0;
		int rowPad = 0;
		if(cellPadding!=null)
		{
			cellPad = Integer.parseInt(cellPadding);
		}
		if(rowPadding!=null)
		{
			rowPad = Integer.parseInt(rowPadding);
		}		
		XFormTableHelper.padTable(table,cellPad,rowPad);
	}
	
	XSwingFormLayout layout = new XSwingFormLayout( table );
	
	container.setLayout( layout );

	Vector xRows = table.getElementsByName("tr");

	for(int loopR=0;loopR<xRows.size();loopR++)
	{
		XmlElement xRow = (XmlElement)xRows.elementAt(loopR);

		Vector xCols = xRow.getElementsByName("td");

		for(int loopC=0;loopC<xCols.size();loopC++)
		{
			XmlElement xCol = (XmlElement)xCols.elementAt(loopC);
			name = xCol.getAttributeValueByName("name");
			if(name==null)
			{
				name = "r"+loopR+"c"+loopC;
				xCol.addAttribute("name",name);
			}

			if(xCol.getElementCount()==1)
			{
				Object data = xCol.getData(0);

				if( data!=null )
				{
					XappComponent comp = XappRootApplication.getComponentFactory().buildXappComponent( container,data,container.getName(),componentStore );

					if(comp!=null)
					{
						container.add(name,comp);

						//System.out.println("Comp:"+comp+" Name:"+name);
					}
				}
				
			}
			else if (xCol.getElementCount()>1)
			{
				Object data = xCol.getElements();

				if( data!=null )
				{
					XappComponent comp = XappRootApplication.getComponentFactory().buildXappComponent( container,data,container.getName(),componentStore );
					
					if(comp!=null)
					{
						container.add(name,comp);
					}
				}
			}
		}
	}

	//System.out.println(table.toXml());

	return layout;
}

public static void buildRow(XSwingForm container,XmlElement xRow,int loopR,XappComponentStore componentStore)
{
	String name = null;

	Vector xCols = xRow.getElementsByName("td");

	for(int loopC=0;loopC<xCols.size();loopC++)
	{
		XmlElement xCol = (XmlElement)xCols.elementAt(loopC);
		name = xCol.getAttributeValueByName("name");
		if(name==null)
		{
			name = "r"+loopR+"c"+loopC;
			xCol.addAttribute("name",name);
		}

		if(xCol.getElementCount()==1)
		{
			Object data = xCol.getData(0);

			if( data!=null )
			{
				XappComponent comp = XappRootApplication.getComponentFactory().buildXappComponent( container,data,container.getName(),componentStore );

				if(comp!=null)
				{
					container.add(name,comp);

					//System.out.println("Comp:"+comp+" Name:"+name);
				}
			}
		}
		else if (xCol.getElementCount()>1)
		{
			Object data = xCol.getElements();

			if( data!=null )
			{
				XappComponent comp = XappRootApplication.getComponentFactory().buildXappComponent( container,data,container.getName(),componentStore );

				if(comp!=null)
				{
					container.add(name,comp);
				}
			}
		}
	}
}

}
