package org.one.stone.soup.xml.file;

import java.io.File;

import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlBuilder;
import org.one.stone.soup.xml.helper.XmlCloner;
import org.one.stone.soup.xml.helper.XmlPathSearch;

public class FilesToXml {
	
	public static final String STATE_UNCHANGED = "unchanged";
	public static final String STATE_CHANGED = "changed";
	public static final String STATE_DELETED = "deleted";
	public static final String STATE_NEW = "new";
	
	private FilesToXml(){}
	
	public static XmlElement directoryToXml(File directory)
	{
		if( !directory.exists() )
		{
			return null;
		}
		else if(directory.isDirectory()==false)
		{
			return fileToXml( directory );
		}
		
		XmlElement xDirectory = new XmlElement("directory");
		xDirectory.addAttribute( "name",directory.getName() );
		xDirectory.addAttribute( "src",directory.getAbsolutePath().replace('\\','/') );
		xDirectory.addAttribute( "timestamp",""+directory.lastModified() );
		
		File[] files = directory.listFiles();
		xDirectory.addAttribute( "files",""+files.length );
		
		for(int loop=0;loop<files.length;loop++)
		{
			if(files[loop].isDirectory())
			{
				xDirectory.addChild( directoryToXml(files[loop]) );
			}
			else
			{
				xDirectory.addChild( fileToXml(files[loop]) );
			}
		}
		
		return xDirectory;
	}
	
	public static XmlElement fileToXml(File file)
	{
		if( !file.exists() || file.isDirectory() )
		{
			return null;
		}
		
		XmlElement xFile = new XmlElement("file");
		xFile.addAttribute( "name",file.getName() );
		xFile.addAttribute( "src",file.getAbsolutePath().replace('\\','/') );
		xFile.addAttribute( "timestamp",""+file.lastModified() );
		xFile.addAttribute( "size",""+file.length() );
		
		return xFile;
	}
	
	public static boolean setDifferences(XmlElement original,XmlElement current)
	{
		boolean hasDifferences = false;
		
		if(original.getName().equals("file"))
		{
			if( !original.getAttributeValueByName("timestamp").equals(current.getAttributeValueByName("timestamp")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				return true;
			}
			if( !original.getAttributeValueByName("size").equals(current.getAttributeValueByName("size")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				return true;
			}
			return false;
		}
		else
		{
			if( !original.getAttributeValueByName("timestamp").equals(current.getAttributeValueByName("timestamp")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
			}
			if( !original.getAttributeValueByName("files").equals(current.getAttributeValueByName("files")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
			}			
		}
		
		for(int loop=0;loop<original.getElementCount();loop++)
		{
			XmlElement testElement = original.getElementByIndex(loop);
			
			XmlPathSearch search = new XmlPathSearch(current);
			XmlElement found = search.findElement( current.getName()+"/"+testElement.getName()+"[@src='"+testElement.getAttributeValueByName("src")+"']" );
			
			if(found==null)
			{
				hasDifferences=true;
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				XmlBuilder.setAttribute(testElement,"state",STATE_DELETED);
			}
			else
			{
				if( setDifferences(testElement,found)==true )
				{
					hasDifferences = true;
				}
			}
		}
		
		for(int loop=0;loop<current.getElementCount();loop++)
		{
			XmlElement testElement = current.getElementByIndex(loop);
			
			XmlPathSearch search = new XmlPathSearch(original);
			XmlElement found = search.findElement( original.getName()+"/"+testElement.getName()+"[@src='"+testElement.getAttributeValueByName("src")+"']" );
			
			if(found==null)
			{
				hasDifferences=true;
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				XmlElement childElement = XmlCloner.getClone(testElement,false);
				original.addChild(childElement);
				XmlBuilder.setAttribute(childElement,"state",STATE_NEW);
				
				setDifferences(childElement,testElement);
			}
		}		
		
		return hasDifferences;
	}
}
